/// <reference types="node" />
import { PackageJson, TsConfigJson } from "type-fest";
declare type MaybePromise<T> = T | Promise<T>;
interface Options {
    /** @default utf-8 */
    encoding: BufferEncoding;
    /**
     * If `false`, FS or JSON errors will be ignored
     * @default true
     *  */
    throws: boolean;
    /** @default "throw" (silent if throws: false) */
    ifFieldIsMissing: "throw" | "skip" | "add";
    /**
     * - throw - throws (silent if throws: false)
     *-  skip - won't call the function
     * - pass - pass the `undefined` value
     * @default "throw"
     * */
    ifFieldIsMissingForSetter: "throw" | "skip" | "pass";
    /**
     * - null - disable formatting
     * - hard - one hard tab \t
     * - number - number of spaces
     * @default "preserve"
     *  */
    tabSize: null | number | "preserve" | "hard";
}
declare type GettersDeep<T extends object> = {
    [K in keyof T]: (oldValue: T[K], json: T) => T[K];
};
export declare type ModifyJsonFileFunction<T extends object> = (path: string, modifyFields: Partial<T | GettersDeep<T>> | ((oldJson: T) => MaybePromise<T>), options?: Options) => Promise<void>;
declare type ModifyJsonFileGenericFunction = <T extends object>(path: string, modifyFields: Partial<T | GettersDeep<T>> | ((oldJson: T) => MaybePromise<T>), options?: Partial<Options>) => Promise<void>;
/** It's just Error */
/**
 * modifies **original** JSON file
 * You can pass generic, that reflects the structure of original JSON file
 *
 * @param modifyFields Fields to merge or callback (can be async). If callback is passed, JSON fields won't be merged.
 */
export declare const modifyJsonFile: ModifyJsonFileGenericFunction;
/**
 * Almost the same is sindresorhus/write-pkg, but with proper typing support and setters for fields
 */
export declare const modifyPackageJsonFile: ModifyJsonFileFunction<PackageJson>;
export declare const modifyTsConfigJsonFile: ModifyJsonFileFunction<TsConfigJson>;
export {};
