"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.modifyTsConfigJsonFile = exports.modifyPackageJsonFile = exports.modifyJsonFile = void 0;
const detect_indent_1 = __importDefault(require("detect-indent"));
const strip_bom_1 = __importDefault(require("strip-bom"));
const parse_json_1 = __importDefault(require("parse-json"));
const graceful_fs_1 = __importDefault(require("graceful-fs"));
/** returns additional info, not only JSON */
const loadJsonFile = async (filePath, { encoding, tabSize }) => {
    const contents = strip_bom_1.default(await graceful_fs_1.default.promises.readFile(filePath, encoding));
    return {
        json: parse_json_1.default(contents, filePath),
        indent: tabSize === "preserve" ?
            detect_indent_1.default(contents).indent :
            tabSize === "hard" ? "\t" : tabSize === null ?
                undefined : " ".repeat(tabSize)
    };
};
/** It's just Error */
// class InnerError extends Error {
//     innerError = true;
// }
/**
 * modifies **original** JSON file
 * You can pass generic, that reflects the structure of original JSON file
 *
 * @param modifyFields Fields to merge or callback (can be async). If callback is passed, JSON fields won't be merged.
 */
const modifyJsonFile = async (path, modifyFields, options) => {
    const { encoding = "utf-8", throws = true, ifFieldIsMissing = "throw", ifFieldIsMissingForSetter = "throw", tabSize = "preserve" } = options || {};
    try {
        let { json, indent } = await loadJsonFile(path, { encoding, tabSize });
        // todo remove restriction or not?
        if (!json || typeof json !== "object" || Array.isArray(json))
            throw new TypeError(`${path}: JSON root type must be object`);
        if (typeof modifyFields === "function") {
            json = await modifyFields(json);
        }
        else {
            for (const [name, value] of Object.entries(modifyFields)) {
                if (!(name in json)) {
                    const isSetter = typeof value === "function";
                    const generalAction = isSetter ? ifFieldIsMissingForSetter : ifFieldIsMissing;
                    if (generalAction === "throw")
                        throw new TypeError(`Property to modify "${name}" is missing in ${path}`);
                    if (generalAction === "skip")
                        continue;
                }
                json[name] = typeof value === "function" ? value(json[name], json) : value;
            }
        }
        await graceful_fs_1.default.promises.writeFile(path, JSON.stringify(json, undefined, indent));
    }
    catch (err) {
        // if (err.innerError) throw new Error(err.message);
        if (throws)
            throw err;
    }
};
exports.modifyJsonFile = modifyJsonFile;
// todo: use read-pkg / write-pkg for normalization
/**
 * Almost the same is sindresorhus/write-pkg, but with proper typing support and setters for fields
 */
exports.modifyPackageJsonFile = exports.modifyJsonFile;
exports.modifyTsConfigJsonFile = exports.modifyJsonFile;
